/*********************************************************/
/* This source code copyright (c) 1991-2001, Aaron Giles */
/* See the Read Me file for licensing information.       */
/* Contact email: mac@aarongiles.com                     */
/*********************************************************/

#ifndef __DIALOGUTILS__
#define __DIALOGUTILS__

enum {
	kIndefProgress = 0x7d8d9dadL,
	kModalProgress = 0x7d8d9daeL
};

extern short HandleDialogLetterKey(DialogPtr theDialog, char theChar);
extern short HandleDialogCancelKey(DialogPtr theDialog);
extern short HandleDialogOKKey(DialogPtr theDialog);
extern Boolean IsDialogWindow(WindowPtr theWindow);
extern void UpdateParamText(StringPtr string1, StringPtr string2, StringPtr string3, 
	StringPtr string4);
extern pascal void PopUpMenuBox(DialogPtr theDialog, short theItem);

Point CenterWindow(WindowPtr theWindow);
void GetMouseGDRect(Rect *theRect);
void MouseCenterRect(Rect *theRect);
short CenterAlert(short id);
short CenterDialog(short id);
Point CenterDialogWhere(short id);
pascal Boolean GenericFilter(DialogPtr theDialog, EventRecord *theEvent, short *itemHit);
pascal void OutlineOK(DialogPtr theDialog, short theItem);
pascal void LineItem(DialogPtr theDialog, short theItem);
void SetItemHandle(DialogPtr theDialog, short theItem, Handle newHandle);
void DisableDControl(DialogPtr theDialog, short theItem);
void EnableDControl(DialogPtr theDialog, short theItem);
void SetDCtlValue(DialogPtr theDialog, short theItem, short theValue);
short GetDCtlValue(DialogPtr theDialog, short theItem);
void SetDIText(DialogPtr theDialog, short theItem, Str255 theText);
void GetDIText(DialogPtr theDialog, short theItem, Str255 theText);
void UpdateDialogItem(DialogPtr theDialog, short theItem);

#endif
